/**
 *  Copyright (C) 2012 Yvon TANGUY

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package org.vono.narau;

import org.vono.narau.Common.DatabseState;
import org.vono.narau.dictionary.DictionaryActivity;
import org.vono.narau.dictionary.DictionaryDB;
import org.vono.narau.download.DownloaderActivity;
import org.vono.narau.exercise.DrillMenuActivity;
import org.vono.narau.kanji.KanjiDB;
import org.vono.narau.learn.reading.LearnReadingMenuActivity;
import org.vono.narau.learn.writing.LearnWritingMenuActivity;
import org.vono.narau.preferences.PreferencesActivity;

import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.os.Bundle;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.widget.Button;
import android.widget.TextView;

/**
 * 
 * <p>
 * <b>Narau</b> 習う (ならう)<br/>
 * to take lessons in/to be taught/to learn (from a teacher)/to study (under a
 * teacher)/to get training in
 * </p>
 * The menu of this application.
 */
public class MenuActivity extends Activity {

	// /**
	// * Log tag
	// */
	// @SuppressWarnings("unused")
	// private static final String TAG = MenuActivity.class.getSimpleName();

	private static final int DOWNLOAD_RESULT_CODE = 123;
	public static final int RESULT_PENDING = 123;

	private enum CurrentActivity {
		NONE, DOWNLOAD, DICTIONARY, LREADING, LWRITING, DRILL
	}

	private static CurrentActivity currentActivity = CurrentActivity.NONE;

	/** Called when the activity is created. */
	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);

		switch (MenuActivity.currentActivity) {
		case DOWNLOAD:
			if (!DownloaderActivity.isDownloadDone()) {
				this.showDownload(false);
			}
			break;
		default:
			QuitThread.stopQuitThread();
			this.doInit();
		}
	}

	private void doInit() {
		final Context context = this.getApplicationContext();
		try {
			Common.init(context);

			boolean haveDatabase = Common.haveDBAvail(Common.DB_MASK_ALL);
			if (haveDatabase) {
				this.showMenuView();
			} else {
				this.showDownload(false);
			}

		} catch (AppException ae) {
			// show the init error view
			super.setContentView(R.layout.menu_init_error);
			TextView tv = (TextView) super.findViewById(R.id.initErrorTextView);
			tv.setText(ae.getMessage());
		}
	}

	/**
	 * Result from the download activity
	 */
	@Override
	protected void onActivityResult(int requestCode, int resultCode, Intent data) {

		if (DOWNLOAD_RESULT_CODE == requestCode) {
			switch (resultCode) {
			case Activity.RESULT_OK:
				MenuActivity.currentActivity = CurrentActivity.NONE;
				Common.clean();
				this.doInit();
				break;
			case Activity.RESULT_CANCELED:
				this.quitApplication(null);
				break;
			case RESULT_PENDING:
				this.finish();
				break;
			}
		}
	}

	/**
	 * 
	 * @param v
	 *            the menu_init_error.xml quit button (initErrorButtonQuit).
	 */
	public void quitApplication(View v) {
		MenuActivity.currentActivity = CurrentActivity.NONE;
		super.finish();
		QuitThread.startQuitThread(4000);
	}

	/**
	 * Show the menu view
	 */
	public void showMenuView() {

		super.setContentView(R.layout.menu);

		if (DictionaryDB.getState() != DatabseState.ok || KanjiDB.getState() != DatabseState.ok) {
			final TextView tvDeprecated = (TextView) super.findViewById(R.id.menuTextViewDeprecated);
			final String strDeprecated = super.getString(R.string.menuDeprecatedDB);
			final String strMissing = super.getString(R.string.menuMissingDB);
			final StringBuilder sb = new StringBuilder();
			if (DictionaryDB.getState() == DatabseState.deprecated) {
				sb.append(String.format(strDeprecated, super.getString(R.string.dbDictionary)));
				sb.append("\n");
			} else if (DictionaryDB.getState() == DatabseState.missing) {
				sb.append(String.format(strMissing, super.getString(R.string.dbDictionary)));
				sb.append("\n");
			}
			if (KanjiDB.getState() == DatabseState.deprecated) {
				sb.append(String.format(strDeprecated, super.getString(R.string.dbKanjiKana)));
				sb.append("\n");
			} else if (KanjiDB.getState() == DatabseState.missing) {
				sb.append(String.format(strMissing, super.getString(R.string.dbKanjiKana)));
				sb.append("\n");
			}
			tvDeprecated.setText(sb);
			tvDeprecated.setVisibility(View.VISIBLE);
		}

		// Disable menu for unavailable DB needed
		int textColor = super.getResources().getColor(android.R.color.darker_gray);
		if (!DictionaryActivity.haveDatabase()) {
			Button button = (Button) super.findViewById(R.id.menuButtonDictionary);
			button.setEnabled(false);
			button.setTextColor(textColor);
		}

		if (!LearnWritingMenuActivity.haveDatabase()) {
			Button button = (Button) super.findViewById(R.id.menuButtonLearnWriting);
			button.setEnabled(false);
			button.setTextColor(textColor);
		}

		if (!LearnReadingMenuActivity.haveDatabase()) {
			Button button = (Button) super.findViewById(R.id.menuButtonLearnReading);
			button.setEnabled(false);
			button.setTextColor(textColor);
		}

		if (!DrillMenuActivity.haveDatabase()) {
			Button button = (Button) super.findViewById(R.id.menuButtonDrillsFromJap);
			button.setEnabled(false);
			button.setTextColor(textColor);

			button = (Button) super.findViewById(R.id.menuButtonDrillsToJap);
			button.setEnabled(false);
			button.setTextColor(textColor);
		}
	}

	private void showDownload(boolean update) {
		MenuActivity.currentActivity = CurrentActivity.DOWNLOAD;
		Intent intent = new Intent(super.getApplicationContext(), DownloaderActivity.class);
		intent.putExtra(DownloaderActivity.UPDATE_PROCESS, update);
		super.startActivityForResult(intent, DOWNLOAD_RESULT_CODE);
	}

	public void showDictionary(View v) {
		Intent intent = new Intent(v.getContext(), DictionaryActivity.class);
		super.startActivity(intent);
	}

	/**
	 * @param v
	 *            The learn writing button.
	 */
	public void showLearnWriting(View v) {
		Intent intent = new Intent(this, LearnWritingMenuActivity.class);
		super.startActivity(intent);
	}

	/**
	 * 
	 * @param v
	 *            the learn reading button.
	 */
	public void showLearnReading(View v) {
		Intent intent = new Intent(this, LearnReadingMenuActivity.class);
		super.startActivity(intent);
	}

	public void showDrillsMenu(View v) {
		Intent intent = new Intent(this, DrillMenuActivity.class);
		if (v.getId() == R.id.menuButtonDrillsFromJap) {
			intent.putExtra(DrillMenuActivity.DRILL_DIRECTION, DrillMenuActivity.DRILL_DIRECTION_FROM);
		} else {
			intent.putExtra(DrillMenuActivity.DRILL_DIRECTION, DrillMenuActivity.DRILL_DIRECTION_TO);
		}

		super.startActivity(intent);
	}

	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		MenuInflater inflater = getMenuInflater();
		inflater.inflate(R.menu.main, menu);
		return true;
	}

	@Override
	public boolean onOptionsItemSelected(MenuItem item) {
		boolean downloading = false;
		if (!downloading) {

			switch (item.getItemId()) {
			case R.id.menuMainQuit:
				MenuActivity.currentActivity = CurrentActivity.NONE;
				QuitThread.startQuitThread();
				super.finish();
				break;
			case R.id.menuMainLicense:
				Intent mIntent = new Intent(this, LicenseActivity.class);
				super.startActivity(mIntent);
				break;
			case R.id.menuMainUpdateDB:
				DownloaderActivity.resetState();
				this.showDownload(true);
				break;
			case R.id.menuMainPreferences:
				mIntent = new Intent(this.getApplicationContext(), PreferencesActivity.class);
				super.startActivity(mIntent);
				break;
			default:
				return super.onOptionsItemSelected(item);
			}
		}
		return true;
	}
}