/**
 *  Copyright (C) 2012 Yvon TANGUY

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package org.vono.narau.kanjidb;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;

import org.tmatesoft.sqljet.core.SqlJetException;
import org.xmlpull.v1.XmlPullParserException;

/**
 * @author vono
 * 
 */
public class Main {
	private static final int NUM_KANJIS = 6660;
	private static final String DEFAULT_DB_NAME = "kanjis.db";

	private static void printUsage() {
		System.out.println("usage:");
		System.out.println("prog [kanjivg_git_dir] <" + KanjiDictReader.DEFAULT_XML_FILE + "> <" + DEFAULT_DB_NAME
				+ ">");
		System.out
				.println("Kanjidict file direct download link: http://www.csse.monash.edu.au/~jwb/kanjidic2/kanjidic2.xml.gz");
		System.out.println("Kanjivg git: https://github.com/KanjiVG/kanjivg.git");
	}

	public static void main(String[] args) {
		String svgDirectoryName = null;
		String dictFilename = null;
		String dbFilename = null;

		long startTime = System.currentTimeMillis();

		if (args.length >= 3) {
			dbFilename = args[2];
		} else {
			dbFilename = DEFAULT_DB_NAME;
		}
		if (args.length >= 2) {
			dictFilename = args[1];
		} else {
			dictFilename = KanjiDictReader.DEFAULT_XML_FILE;
		}
		if (args.length >= 1) {
			svgDirectoryName = args[0];
		} else {
			printUsage();
			return;
		}

		File svgDirectory = new File(svgDirectoryName + "/kanji");
		if (!svgDirectory.isDirectory() || !svgDirectory.canRead()) {
			printUsage();
			return;
		} else {
			System.out.println("Found svg directory " + svgDirectory.getAbsolutePath());
		}

		File dictFile = new File(dictFilename);
		if (!dictFile.isFile() || !dictFile.canRead()) {
			printUsage();
			return;
		} else {
			System.out.println("Found dictionary " + dictFile.getName());
		}

		File dbFile = new File(dbFilename);
		if (dbFile.exists()) {
			System.out.println("delete old db " + dbFile.getAbsolutePath());
			dbFile.delete();
		} else {
			System.out.println("Target DB file: " + dbFile.getName());
		}

		KanjiVGReader readerVG = null;
		KanjiDictReader readerDict = null;
		HashMap<Integer, Kanji> mapKanjis = new HashMap<Integer, Kanji>(NUM_KANJIS);

		try {
			File[] svgFiles = svgDirectory.listFiles();
			final int numSVGFiles = svgFiles.length;
			System.out.println("reading " + numSVGFiles + " svg files");
			readerVG = new KanjiVGReader(mapKanjis);
			boolean error = false;
			for (int i = 0; i < numSVGFiles; i++) {
				final File svgFile = svgFiles[i];

				if (!svgFile.getName().contains("-")) {
					if (!readerVG.read(svgFile)) {
						i = numSVGFiles;
						error = true;
					}
				}
			}

			if (!error) {
				readerDict = new KanjiDictReader(dictFile, mapKanjis);
				readerDict.read();

				ArrayList<Kanji> listKanji = new ArrayList<Kanji>(mapKanjis.values());
				Collections.sort(listKanji);

				KanjiDBWriter writer = new KanjiDBWriter(listKanji);
				writer.createDB(dbFile);
			}
		} catch (FileNotFoundException fnfe) {
			fnfe.printStackTrace();
		} catch (XmlPullParserException xppe) {
			xppe.printStackTrace();
		} catch (IOException ioe) {
			ioe.printStackTrace();
		} catch (SqlJetException sje) {
			sje.printStackTrace();
		}

		long endTime = System.currentTimeMillis();
		long sec = (endTime - startTime) / 1000;
		long msec = (endTime - startTime) % 1000;
		System.out.println("create db time: " + sec + "." + msec + "s");

	}

}
